#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_db_secret_engine_config
version_added: "3.17.8"
short_description: Hashicorp Vault database secrets engine config
description:
    - Module to configure a database secrets engine
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: database
    state:
        description:
            - should configuration be present or absent
        default: present
    config_file:
        description:
            - Optional location of file containing relevant db configuration info. use either this or the following
              ansible params in your play
    connection_details:
        description:
            - root level database credential for example username, password, connection_url.
    plugin_name:
        description:
            - name of database plugin used. see out of the box list at
              https://www.vaultproject.io/docs/secrets/databases/index.html
    allowed_roles:
        description:
            - list of the roles allowed to use this connection. Defaults to empty (no roles), if contains a "*" any role
              can use this connection.
        default: []
    verify_connection:
        description:
            - Specifies if the connection is verified during initial configuration. Defaults to true.
        default: true
    root_credentials_rotate_statements:
        description:
            - Specifies the database statements to be executed to rotate the root user's credentials. See the plugin's
              API page for more information on support and formatting for this parameter.
        default: []
        aliases: [ root_rotation_statements ]
    password_policy:
        description:
            - The name of the password policy to use when generating passwords for this database. If not specified,
              this will use a default policy defined as 20 characters with at least 1 uppercase, 1 lowercase, 1 number,
              and 1 dash character.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_db_secret_engine_config:
        name: test
        plugin_name: "postgresql-database-plugin" #https://www.vaultproject.io/docs/secrets/databases/index.html
        allowed_roles: ["my-role"]
        connection_details:
            username: "myuser@dbname"
            password: "P@ssw0rd"
            connection_url: "postgresql://{{'{{username}}'}}:{{'{{password}}'}}@blergh-db.com:5230"
        state: "present

    - hashivault_db_secret_engine_config:
        name: test
        config_file: /users/drewbuntu/my-db-config.json
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['mount_point'] = dict(required=False, type='str', default='database')
    argspec['config_file'] = dict(required=False, type='str', default=None)
    argspec['plugin_name'] = dict(required=False, type='str')
    argspec['allowed_roles'] = dict(required=False, type='list', default=[])
    argspec['root_credentials_rotate_statements'] = dict(required=False, type='list',
                                                         aliases=['root_rotation_statements'], default=[])
    argspec['verify_connection'] = dict(required=False, type='bool', default=True)
    argspec['connection_details'] = dict(required=True, type='dict', no_log=True)
    argspec['password_policy'] = dict(required=False, type='str', no_log=True, default='')
    required_one_of = [['config_file', 'connection_details']]

    module = hashivault_init(argspec, supports_check_mode=True, required_one_of=required_one_of)
    result = hashivault_db_secret_engine_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_db_secret_engine_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    config_file = params.get('config_file')
    mount_point = params.get('mount_point').strip('/')
    state = params.get('state')
    name = params.get('name')
    desired_state = dict()

    # if config_file is set value from file
    # else set from passed args
    if config_file:
        desired_state = json.loads(open(params.get('config_file'), 'r').read())
    else:
        desired_state['plugin_name'] = params.get('plugin_name')
        desired_state['allowed_roles'] = params.get('allowed_roles')
        desired_state['verify_connection'] = params.get('verify_connection')
        desired_state['password_policy'] = params.get('password_policy')
        desired_state['root_rotation_statements'] = params.get('root_credentials_rotate_statements')
        connection_details = params.get('connection_details')
        desired_state.update(connection_details)
        del connection_details["password"]

    # not a required param but must ensure a value for current vs desired object comparison
    if 'root_rotation_statements' not in desired_state:
        desired_state['root_rotation_statements'] = []

    exists = False
    current_state = {}
    try:
        current_state = client.secrets.database.read_connection(name=name, mount_point=mount_point)
        exists = True
    except Exception:
        # does not exist
        pass

    changed = False
    if exists:
        if state == 'absent':
            changed = True
        else:
            for k, v in current_state.get('data', {}).items():
                # connection_url is passed as a nested item
                if k == 'connection_details':
                    if v['username'] != desired_state['username']:
                        changed = True
                    if v['connection_url'] != desired_state['connection_url']:
                        changed = True
                elif k == 'root_credentials_rotate_statements':
                    if v != desired_state['root_rotation_statements']:
                        changed = True
                elif v != desired_state[k]:
                    changed = True
    elif state == 'present':
        changed = True

    # if configs dont match and checkmode is off, complete the change
    if changed and state == 'present' and not module.check_mode:
        client.secrets.database.configure(name=name, mount_point=mount_point, **desired_state)
    elif changed and state == 'absent' and not module.check_mode:
        client.secrets.database.delete_connection(name=name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_ldap_group
version_added: "3.18.3"
short_description: Hashicorp Vault LDAP group configuration module
description:
    - Module to configure LDAP groups in Hashicorp Vault.
options:
    mount_point:
        description:
            - location where this method/backend is mounted. also known as "path"
        default: ldap
    name:
        description:
            - name of the group
        default: None
    policies:
        description:
            - policies to be tied to the group
        default: None
    state:
        description:
            - whether create/update or delete the entity
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_ldap_group:
        name: 'my-group'
        policies:
            - 'my-policy'
        token: "{{ vault_token }}"
        url: "{{ vault_url }}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str', default=None)
    argspec['mount_point'] = dict(required=False, type='str', default='ldap')
    argspec['policies'] = dict(required=False, type='list', default=[])
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_ldap_group(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def hashivault_ldap_group_update(group_details, client, group_name, group_policies, mount_point):
    changed = False

    # existing policies
    if group_details['policies'] is not None:
        if set(group_details['policies']) != set(group_policies):
            changed = True
    # new policies and none existing
    elif len(group_policies) > 0:
        changed = True

    if changed:
        try:
            response = client.auth.ldap.create_or_update_group(
                name=group_name,
                policies=group_policies,
                mount_point=mount_point
            )
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        if response.status_code == 204:
            return {'changed': True}
        return {'changed': True, 'data': response}
    return {'changed': False}


def hashivault_ldap_group_create_or_update(params):
    client = hashivault_auth_client(params)
    group_name = params.get('name')
    mount_point = params.get('mount_point')
    group_policies = params.get('policies')
    try:
        group_details = client.auth.ldap.read_group(name=group_name, mount_point=mount_point)
    except Exception:
        client.auth.ldap.create_or_update_group(
            name=group_name,
            policies=group_policies,
            mount_point=mount_point
        )
        return {'changed': True}
    return hashivault_ldap_group_update(group_details['data'], client, group_name=group_name,
                                        group_policies=group_policies,
                                        mount_point=mount_point)


def hashivault_ldap_group_delete(params):
    client = hashivault_auth_client(params)
    group_name = params.get('name')

    try:
        client.auth.ldap.read_group(name=group_name)
    except Exception:
        return {'changed': False}
    client.auth.ldap.delete_group(name=group_name)
    return {'changed': True}


@hashiwrapper
def hashivault_ldap_group(params):
    state = params.get('state')
    if state == 'present':
        return hashivault_ldap_group_create_or_update(params)
    elif state == 'absent':
        return hashivault_ldap_group_delete(params)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_policy_list
version_added: "2.1.0"
short_description: Hashicorp Vault policy list module
description:
    - Module to list policies in Hashicorp Vault.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_policy_list:
      register: 'vault_policy_list'
    - debug: msg="Policies are {{vault_policy_list.policy}}"
'''


def main():
    argspec = hashivault_argspec()
    module = hashivault_init(argspec)
    result = hashivault_policy_list(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_policy_list(params):
    client = hashivault_auth_client(params)
    current_policies = client.sys.list_policies()
    if isinstance(current_policies, dict):
        current_policies = current_policies.get('data', current_policies)
        current_policies = current_policies.get('policies', current_policies)
    return {'policies': current_policies}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import is_state_changed
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_role
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Create/Update/Delete Role
description:
    - This module creates or updates the role definition.
    - Note that the `allowed_domains`, `allow_subdomains`, `allow_glob_domains`, and `allow_any_name` attributes are
      additive; between them nearly and across multiple roles nearly any issuing policy can be accommodated.
      `server_flag`, `client_flag`, and `code_signing_flag` are additive as well.
    - If a client requests a certificate that is not allowed by the CN policy in the role, the request is denied.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    name:
        recuired: true
        description:
            - Specifies the name of the role to create.
    role_file:
        description:
            - file with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
    state:
        description:
            - Do you want for this config to be present or absent
        choices: ["present", "absent"]
        default: present
    config:
        type: dict
        description:
            - Collection of properties from pki role
              U(https://www.vaultproject.io/api-docs/secret/pki#create-update-role)
        suboptions:
            ttl:
                type: str
                description:
                    - Specifies the Time To Live value provided as a string duration with time suffix.
                    - Hour is the largest suffix.
                    - If not set, uses the system default value or the value of `max_ttl`, whichever is shorter.
            max_ttl:
                type: str
                description:
                    - Specifies the maximum Time To Live value provided as a string duration with time suffix.
                    - Hour is the largest suffix.
                    - If not set, defaults to the system maximum lease TTL.
            allow_localhost:
                type: bool
                default: true
                description:
                    - Specifies if clients can request certificates for `localhost` as one of the requested common
                      names.
                    - This is useful for testing and to allow clients on a single host to talk securely.
            allowed_domains:
                type: list
                description:
                    - Specifies the domains of the role.
                    - This is used with the `allow_bare_domains` and `allow_subdomains` options.
            allowed_domains_template:
                type: bool
                default: false
                description:
                    - When set, `allowed_domains` may contain templates, as with
                      [ACL Path Templating](https://www.vaultproject.io/docs/concepts/policies).
            allow_bare_domains:
                type: bool
                default: false
                description:
                    - Specifies if clients can request certificates matching the value of the actual domains themselves.
                    - e.g. if a configured domain set with `allowed_domains` is `example.com`, this allows clients to
                      actually request a certificate containing the name `example.com` as one of the DNS values on the
                      final certificate. In some scenarios, this can be considered a security risk.
            allow_subdomains:
                type: bool
                default: false
                description:
                    - Specifies if clients can request certificates with CNs that are subdomains of the CNs allowed by
                      the other role options. This includes wildcard subdomains.
                    - For example, an `allowed_domains` value of `example.com` with this option set to true will allow
                      `foo.example.com` and `bar.example.com` as well as `*.example.com`.
                    - This is redundant when using the `allow_any_name` option.
            allow_glob_domains:
                type: bool
                default: false
                description:
                    - Allows names specified in `allowed_domains` to contain glob patterns (e.g. `ftp*.example.com`)
                    - Clients will be allowed to request certificates with names matching the glob patterns.
            allow_any_name:
                type: bool
                default: false
                description:
                    - Specifies if clients can request any CN
                    - Useful in some circumstances, but make sure you understand whether it is appropriate for your
                      installation before enabling it.
            enforce_hostnames:
                type: bool
                default: true
                description:
                    - Specifies if only valid host names are allowed for CNs, DNS SANs, and the host part of email
                      addresses.
            allow_ip_sans:
                type: bool
                default: true
                description:
                    - Specifies if clients can request IP Subject Alternative Names
                    - No authorization checking is performed except to verify that the given values are valid IP
                      addresses.
            allowed_uri_sans:
                type: list
                description:
                    - Defines allowed URI Subject Alternative Names
                    - No authorization checking is performed except to verify that the given values are valid URIs
                    - Values can contain glob patterns (e.g. `spiffe://hostname/*`).
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            allowed_other_sans:
                type: type
                description:
                    - Defines allowed custom OID/UTF8-string SANs
                    - Each item of the list has the same format as OpenSSL `<oid>;<type>:<value>`, but the only valid
                      type is `UTF8` or `UTF-8`.
                    - The `value` part of an element may be a `*` to allow any value with that OID
                    - Alternatively, specifying a single `*` will allow any `other_sans` input. `server_flag`
                      `(bool)` Specifies if certificates are flagged for server use.
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            server_flag:
                type: bool
                default: true
                description:
                    - Specifies if certificates are flagged for server use.
            client_flag:
                type: bool
                default: true
                description:
                    - Specifies if certificates are flagged for client use.
            code_signing_flag:
                type: bool
                default: false
                description:
                    - Specifies if certificates are flagged for code signing use.
            email_protection_flag:
                type: bool
                default: false
                description:
                    - Specifies if certificates are flagged for email protection use.
            key_type:
                type: str
                default: "rsa"
                description:
                    - Specifies the type of key to generate for generated private keys and the type of key expected for
                      submitted CSRs
                    - Currently, `rsa` and `ec` are supported, or when signing CSRs `any` can be specified to allow
                      keys of either type and with any bit size (subject to > 1024 bits for RSA keys).
            key_bits:
                type: int
                default: 2048
                description:
                    - Specifies the number of bits to use for the generated keys
                    - This will need to be changed for `ec` keys, e.g., 224 or 521.
            key_usage:
                type: list
                default: ["DigitalSignature", "KeyAgreement", "KeyEncipherment"]
                description:
                    - Specifies the allowed key usage constraint on issued certificates
                    - Valid values can be found at U(https://golang.org/pkg/crypto/x509/#KeyUsage) - simply drop the
                      `KeyUsage` part of the value
                    - Values are not case-sensitive
                    - To specify no key usage constraints, set this to an empty list.
            ext_key_usage:
                type: list
                description:
                    - Specifies the allowed extended key usage constraint on issued certificates
                    - Valid values can be found at U(https://golang.org/pkg/crypto/x509/#ExtKeyUsage) - simply drop the
                      `ExtKeyUsage` part of the value
                    - Values are not case-sensitive
                    - To specify no key usage constraints, set this to an empty list.
            ext_key_usage_oids:
                type: list
                description:
                    - List of extended key usage oids.
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            use_csr_common_name:
                type: bool
                default: true
                description:
                    - When used with the CSR signing endpoint, the common name in the CSR will be used instead of taken
                      from the JSON data
                    - This does `not` include any requested SANs in the CSR; use `use_csr_sans` for that.
            use_csr_sans:
                type: bool
                default: true
                description:
                    - When used with the CSR signing endpoint, the subject alternate names in the CSR will be used
                      instead of taken from the JSON data
                    - This does `not` include the common name in the CSR; use `use_csr_common_name` for that.
            ou:
                type: list
                description:
                    - Specifies the OU (OrganizationalUnit) values in the subject field of issued certificates.
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            organization:
                type: list
                description:
                    - Specifies the O (Organization) values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            country:
                type: list
                description:
                    - Specifies the C (Country) values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            locality:
                type: list
                description:
                    - Specifies the L (Locality) values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            province:
                type: list
                description:
                    - Specifies the ST (Province) values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            street_address:
                type: list
                description:
                    - Specifies the Street Address values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            postal_code:
                type: list
                description:
                    - Specifies the Postal Code values in the subject field of issued certificates
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            serial_number:
                type: str
                description:
                    - Specifies the Serial Number, if any
                    - Otherwise Vault will generate a random serial for you
                    - If you want more than one, specify alternative names in the alt_names map using OID 2.5.4.5.
            generate_lease:
                type: bool
                default: false
                description:
                    - Specifies if certificates issued/signed against this role will have Vault leases attached to them
                    - Certificates can be added to the CRL by `vault revoke <lease_id>` when certificates are
                      associated with leases
                    - It can also be done using the `pki/revoke` endpoint
                    - However, when lease generation is disabled, invoking `pki/revoke` would be the only way to add
                      the certificates to the CRL.
            no_store:
                type: bool
                default: false
                description:
                    - If set, certificates issued/signed against this role will not be stored in the storage backend
                    - This can improve performance when issuing large numbers of certificates
                    - However, certificates issued in this way cannot be enumerated or revoked, so this option is
                      recommended only for certificates that are non-sensitive, or extremely short-lived
                    - This option implies a value of `false` for `generate_lease`.
            require_cn:
                type: bool
                default: true
                description:
                    - If set to false, makes the `common_name` field optional while generating a certificate.
            policy_identifiers:
                type: list
                description:
                    - A comma-separated string or list of policy OIDs.
                    - Although this parameter could take a string with comma-delimited items, it's highly advised to
                      not do so as it would break idempotency.
            basic_constraints_valid_for_non_ca:
                type: bool
                default: false
                description:
                    - Mark Basic Constraints valid when issuing non-CA certificates.
            not_before_duration:
                type: duration
                default: "30s"
                description:
                    - Specifies the duration by which to backdate the NotBefore property.
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_role:
        name: tester
        config:
            allow_subdomains: true

    - hashivault_pki_role:
        name: tester
        role_file: "/opt/vault/etc/roles/pki-tester.json"
        state: "present"
'''
normalize = {'list': list, 'str': str, 'dict': dict, 'bool': bool, 'int': int, 'duration': str}


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['mount_point'] = dict(required=False, type='str', default='pki')
    argspec['role_file'] = dict(required=False, type='str')
    argspec['config'] = dict(required=False, type='dict')

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_role(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_role(module):
    params = module.params
    client = hashivault_auth_client(params)

    name = params.get('name').strip('/')
    mount_point = params.get('mount_point').strip('/')
    state = params.get('state')
    role_file = params.get('role_file')
    config = params.get('config')

    desired_state = {}
    exists = False

    if role_file:
        import json
        desired_state = json.loads(open(role_file, 'r').read())
    elif config:
        import yaml
        doc = yaml.safe_load(DOCUMENTATION)
        args = doc.get('options').get('config').get('suboptions').items()
        for key, value in args:
            arg = config.get(key)
            if arg is not None:
                try:
                    desired_state[key] = normalize[value.get('type')](arg)
                except Exception:
                    return {'changed': False, 'failed': True,
                            'msg': 'config item \'{}\' has wrong data format'.format(key)}

    changed = False
    try:
        current_state = client.secrets.pki.read_role(name=name, mount_point=mount_point).get('data')
    except Exception:
        current_state = {}
    if current_state:
        exists = True

    if (exists and state == 'absent') or (not exists and state == 'present'):
        changed = True

    # compare current_state to desired_state
    if exists and state == 'present' and not changed:
        changed = is_state_changed(desired_state, current_state)

    # make the changes!
    if changed and state == 'present' and not module.check_mode:
        client.secrets.pki.create_or_update_role(name=name, mount_point=mount_point, extra_params=desired_state)

    elif changed and state == 'absent' and not module.check_mode:
        client.secrets.pki.delete_role(name=name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_set_signed
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Set Signed Intermediate
description:
    - This module allows submitting the signed CA certificate corresponding to a private key generated via
      `intermediate` type in `hashivault_pki_ca` module.
    - The certificate should be submitted in PEM format.
options:
    certificate:
        recuired: true
        type: str
        description:
            - Specifies the name of the role to create.
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - name: Generate Intermediate
      hashivault_pki_ca:
        mount_point: "{{mount_inter}}"
        common_name: my common name
        kind: intermediate
      register: response
    - name: List Certificates
      hashivault_pki_cert_list:
        mount_point: "{{mount_inter}}"
      register: list
    - name: Sign Intermediate
      hashivault_pki_cert_sign:
        mount_point: "{{mount_root}}"
        csr: "{{response.data.csr}}"
        common_name: my common name
        type: intermediate
      register: response
    - name: Set Signed Intermediate
      hashivault_pki_set_signed:
        mount_point: "{{mount_inter}}"
        certificate: "{{ response.data.certificate }}\n{{ response.data.issuing_ca }}"

'''


def main():
    argspec = hashivault_argspec()
    argspec['certificate'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_set_signed(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_set_signed(module):
    params = module.params
    client = hashivault_auth_client(params)

    certificate = params.get('certificate')
    mount_point = params.get('mount_point').strip('/')

    result = {"changed": False, "rc": 0}
    try:
        result['changed'] = client.secrets.pki.set_signed_intermediate(certificate=certificate,
                                                                       mount_point=mount_point).ok
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_generate_root_cancel
version_added: "3.14.0"
short_description: Hashicorp Vault generate_root cancel module
description:
    - Module to cancel generation of root token of Hashicorp Vault.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_generate_root_cancel:
'''


def main():
    argspec = hashivault_argspec()
    module = hashivault_init(argspec)
    result = hashivault_generate_root_cancel(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_generate_root_cancel(params):
    client = hashivault_client(params)
    # Check if generate_root is on-going & return when generate_root not in progress
    status = client.generate_root_status
    if not status['started']:
        return {'changed': False}
    return {'status': client.cancel_generate_root(), 'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashivault_read

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_read
version_added: "0.1"
short_description: Hashicorp Vault read module
description:
    - Module to read to Hashicorp Vault.
options:
    version:
        description:
            - version of the kv engine (int)
        default: 1
    mount_point:
        description:
            - secret mount point
        default: secret
    secret:
        description:
            - secret to read.
    key:
        description:
            - secret key to read.
    secret_version:
        description:
            - secret version to read kv2 only.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_read:
        secret: 'giant'
        key: 'fie'
      register: 'fie'
    - debug: msg="Value is {{fie.value}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['version'] = dict(required=False, type='int', default=1)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(required=True, type='str')
    argspec['key'] = dict(required=False, type='str')
    argspec['default'] = dict(required=False, default=None, type='str')
    argspec['secret_version'] = dict(required=False, type='int')
    module = hashivault_init(argspec)
    result = hashivault_read(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_consul_secret_engine_role
version_added: "4.4.7"
short_description: Hashicorp Vault database secret engine role
description:
    - Module to define a database role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: consul
    name:
        description:
            - Specifies the name of an existing role against which to create this Consul credential
    token_type:
        description:
            - Specifies the type of token to create when using this role. choices: client, management
        default: client
    policy:
        description:
            - Specifies the base64 encoded ACL policy. This is required unless the token_type is management
        default: ""
    policies:
        description:
            - The list of policies to assign to the generated token. This is only available in Consul 1.4 and greater.
        default: []
    local:
        description:
            - Indicates that the token should not be replicated globally and instead be local to the current datacenter.
            - Only available in Consul 1.4 and greater.
        default: false
    ttl:
        description:
            - Specifies the TTL for this role.
            - This is provided as a string duration with a time suffix like "30s" or "1h" or as seconds.
            - If not provided, the default Vault TTL is used.
        default: ""
    max_ttl:
        description:
            - Specifies the max TTL for this role.
            - This is provided as a string duration with a time suffix like "30s" or "1h" or as seconds.
            - If not provided, the default Vault Max TTL is used.
        default: ""
    state:
        description:
            - state of the object. choices: present, absent
        default: present
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
      hashivault_consul_secret_engine_role:
        name: tester
        policy: pocketknife
        state: present
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='consul')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['token_type'] = dict(required=False, type='str', default='client', choices=['client', 'management'])
    argspec['policy'] = dict(required=False, type='str')
    argspec['policies'] = dict(required=False, type='list')
    argspec['local'] = dict(required=False, type='bool', default=False)
    argspec['ttl'] = dict(required=False, type='int', default=0)
    argspec['max_ttl'] = dict(required=False, type='int', default=0)

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_consul_secret_engine_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_consul_secret_engine_role(module):
    params = module.params
    client = hashivault_auth_client(params)
    name = params.get('name').strip('/')
    mount_point = params.get('mount_point').strip('/')
    state = params.get('state')

    desired_state = dict()
    desired_state['token_type'] = params.get('token_type')
    desired_state['policy'] = params.get('policy')
    desired_state['policies'] = params.get('policies')
    desired_state['local'] = params.get('local')
    desired_state['ttl'] = params.get('ttl')
    desired_state['max_ttl'] = params.get('max_ttl')

    if state == "present" and desired_state['token_type'] == "client" and \
            (not desired_state['policy'] and not desired_state['policies']):
        return {'failed': True, 'msg': 'provide policy or policies for client token', 'rc': 1}

    if (mount_point + "/") not in client.sys.list_mounted_secrets_engines()['data']:
        return {'failed': True, 'msg': 'secret engine is not enabled', 'rc': 1}

    exists = False
    changed = False
    try:
        current_state = client.secrets.consul.read_role(name, mount_point=mount_point)['data']
        exists = True
        if state == 'absent':
            changed = True
        for key in desired_state.keys():
            if key not in current_state:
                continue
            elif desired_state[key] != current_state[key]:
                changed = True
    except Exception:
        pass

    if not exists and state == 'present':
        changed = True

    if not changed or module.check_mode:
        return {'changed': changed}

    if state == 'present':
        client.secrets.consul.create_or_update_role(name, mount_point=mount_point, **desired_state)
    elif state == 'absent':
        client.secrets.consul.delete_role(name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_identity_entity
version_added: "3.13.0"
short_description: Hashicorp Vault entity create module
description:
    - Module to manage identity entity in Hashicorp Vault.
options:
    name:
        description:
            - entity name to create or update.
    id:
        description:
            - entity id to update.
    metadata:
        description:
            - metadata to be associated with entity
    disabled:
        description:
            - whether the entity is disabled
    policies:
        description:
            - entity policies.
    state:
        description:
            - whether create/update or delete the entity
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_identity_entity:
      name: 'bob'
      policies: 'bob'
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=False, type='str', default=None)
    argspec['id'] = dict(required=False, type='str', default=None)
    argspec['metadata'] = dict(required=False, type='dict', default=None)
    argspec['disabled'] = dict(required=False, type='bool', default=None)
    argspec['policies'] = dict(required=False, type='list', default=None)
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_identity_entity(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def hashivault_identity_entity_update(entity_details, client, entity_id, entity_name, entity_metadata, entity_disabled,
                                      entity_policies):
    if entity_metadata is None:
        entity_metadata = entity_details['metadata']
    if entity_policies is None:
        entity_policies = entity_details['policies']
    if entity_disabled is None:
        entity_disabled = entity_details['disabled']

    if entity_details['name'] != entity_name or entity_details['disabled'] != entity_disabled or \
            entity_details['metadata'] != entity_metadata or \
            set([] if entity_details['policies'] is None else entity_details['policies']) != set(entity_policies):
        try:
            client.secrets.identity.update_entity(
                entity_id=entity_id,
                name=entity_name,
                metadata=entity_metadata,
                policies=entity_policies,
                disabled=entity_disabled
            )
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        return {'changed': True}
    return {'changed': False}


def hashivault_identity_entity_create_or_update(params):
    client = hashivault_auth_client(params)
    entity_name = params.get('name')
    entity_id = params.get('id')
    entity_metadata = params.get('metadata')
    entity_disabled = params.get('disabled')
    entity_policies = params.get('policies')

    if entity_id is not None:
        try:
            entity_details = client.secrets.identity.read_entity(entity_id=entity_id)
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        return hashivault_identity_entity_update(entity_details['data'], client, entity_name, entity_id,
                                                 entity_metadata, entity_disabled, entity_policies)
    elif entity_name is not None:
        try:
            entity_details = client.secrets.identity.read_entity_by_name(name=entity_name)
        except Exception:
            response = client.secrets.identity.create_or_update_entity_by_name(
                name=entity_name,
                metadata=entity_metadata,
                policies=entity_policies,
                disabled=entity_disabled
            )
            return {'changed': True, 'data': response['data']}
        return hashivault_identity_entity_update(entity_details['data'], client, entity_name=entity_name,
                                                 entity_id=entity_details['data']['id'],
                                                 entity_metadata=entity_metadata,
                                                 entity_disabled=entity_disabled, entity_policies=entity_policies)
    return {'failed': True, 'msg': "Either name or id must be provided"}


def hashivault_identity_entity_delete(params):
    client = hashivault_auth_client(params)
    entity_id = params.get('id')
    entity_name = params.get('name')

    if entity_id is not None:
        try:
            client.secrets.identity.read_entity(entity_id=entity_id)
        except Exception:
            return {'changed': False}
        client.secrets.identity.delete_entity(entity_id=entity_id)
        return {'changed': True}
    elif entity_name is not None:
        try:
            client.secrets.identity.read_entity_by_name(name=entity_name)
        except Exception:
            return {'changed': False}
        client.secrets.identity.delete_entity_by_name(name=entity_name)
        return {'changed': True}
    return {'failed': True, 'msg': "Either name or id must be provided"}


@hashiwrapper
def hashivault_identity_entity(params):
    state = params.get('state')
    if state == 'present':
        return hashivault_identity_entity_create_or_update(params)
    elif state == 'absent':
        return hashivault_identity_entity_delete(params)
    else:
        return {'failed': True, 'msg': 'Unknown state'}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['deprecated'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_policy_set_from_file
version_added: "2.1.0"
short_description: Hashicorp Vault policy set from a file module
description:
    - Module to set a policy from a file in Hashicorp Vault. Use hashivault_policy_set instead.
options:
    name:
        description:
            - policy name.
    rules_file:
        description:
            - policy rules file.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_policy_set_from_file:
      rules_file: /path/to/policy_file.hcl
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['rules_file'] = dict(required=True, type='str')
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_policy_set_from_file(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_policy_set_from_file(module):
    params = module.params
    client = hashivault_auth_client(params)
    name = params.get('name')
    rules = open(params.get('rules_file'), 'r').read()
    changed = False
    exists = False
    current = str()

    # does policy exit
    try:
        current = client.get_policy(name)
        exists = True
    except Exception:
        if module.check_mode:
            changed = True
        else:
            return {'failed': True, 'msg': 'auth mount is not enabled', 'rc': 1}

    # does current policy match desired
    if exists:
        if current != rules:
            changed = True

    if exists and changed and not module.check_mode:
        client.sys.create_or_update_policy(name, rules)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

from hvac.exceptions import InvalidPath


ANSIBLE_METADATA = {'status': ['deprecated'], 'alternative': 'Use M(hashivault_aws_auth_role) instead.',
                    'why': 'This module does not fit the standard pattern',
                    'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_aws_ec2_role_create
version_added: "3.9.8"
short_description: Hashicorp Vault aws ec2 create role module
description:
    - Module to create a aws ec2 backed vault role Use hashivault_aws_auth_role instead.
options:
    name:
        description:
            - role name.
    bound_ami_id:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they should be using\
             the AMI ID specified"
    bound_vpc_id:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they be associated\
             with the VPC ID that matches the value"
    policies:
        description:
            - policies for the role.
    inferred_entity_type:
        description:
            - Instructs Vault to turn on inferencing. The only valid value is ec2_instance
    auth_type:
        description:
            -  auth type permitted for this role. Valid choices are ec2 and iam
    bound_account_id:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they should be using\
             the account ID"
    bound_iam_instance_profile_arn:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they must be\
             associated with an IAM instance profile"
    bound_iam_role_arn:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they must match the\
             IAM role ARN"
    bound_subnet_id:
        description:
            - "defines a constraint on the EC2 instances that can perform the login operation that they be associated\
             with the subnet ID"
    allow_instance_migration:
        description:
            - if set to true, allows migration of the underlying instance where the client resides.
    disallow_reauthentication:
        description:
            - If set to true, only allows a single token to be granted per instance ID.
    resolve_aws_unique_ids:
        description:
            - If set to true, the bound_iam_principal_arn is resolved to an AWS Unique ID for the bound principal ARN.
    token_max_ttl:
        description:
            - The maximum allowed lifetime of tokens issued using this role, provided as a number of seconds
    token_ttl:
        description:
            - The TTL period of tokens issued using this role, provided as a number of seconds
    mount_point:
        description:
            - location where this auth_method will be mounted. also known as "path"
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_aws_ec2_role_create:
        name: myrole
        auth_type: iam
        inferred_entity_type: ec2_instance
        inferred_aws_region: eu-west-1
        bound_iam_role_arn: arn:aws:iam::12345678:root/ec2-role
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['bound_ami_id'] = dict(required=False, type='str')
    argspec['bound_vpc_id'] = dict(required=False, type='str')
    argspec['inferred_entity_type'] = dict(required=True, type='str')
    argspec['inferred_aws_region'] = dict(required=False, type='str')
    argspec['auth_type'] = dict(required=True, type='str')
    argspec['bound_account_id'] = dict(required=False, type='str')
    argspec['bound_iam_role_arn'] = dict(required=False, type='str')
    argspec['bound_iam_instance_profile_arn'] = dict(required=False, type='str')
    argspec['bound_ec2_instance_id'] = dict(required=False, type='str')
    argspec['bound_subnet_id'] = dict(required=False, type='str')
    argspec['allow_instance_migration'] = dict(required=False, type='bool')
    argspec['disallow_reauthentication'] = dict(required=False, type='bool')
    argspec['resolve_aws_unique_ids'] = dict(required=False, type='bool')
    argspec['token_max_ttl'] = dict(required=False, type='int')
    argspec['token_ttl'] = dict(required=False, type='int')
    argspec['mount_point'] = dict(required=False, default='aws', type='str')
    module = hashivault_init(argspec)
    result = hashivault_aws_ec2_role_create(module.params)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_aws_ec2_role_create(params):
    args = [
        'bound_ami_id',
        'bound_vpc_id',
        'inferred_entity_type',
        'inferred_aws_region',
        'bound_account_id',
        'bound_iam_role_arn',
        'bound_iam_instance_profile_arn',
        'auth_type'
        'bound_ec2_instance_id',
        'allow_instance_migration',
        'disallow_reauthentication',
        'token_ttl',
        'token_max_ttl',
    ]
    name = params.get('name')
    policies = params.get('policies')
    mount_point = params.get('mount_point').strip('/')
    client = hashivault_auth_client(params)
    kwargs = {
        'policies': policies,
    }
    for arg in args:
        value = params.get(arg)
        if value is not None:
            kwargs[arg] = value

    result = client.sys.list_auth_methods()
    backends = result.get('data', result)
    if (mount_point + "/") not in backends:
        return {'failed': True, 'msg': 'aws auth backend is not enabled', 'rc': 1}

    try:
        if client.get_role(name, mount_point):
            return {'changed': False}
    except InvalidPath:
        client.create_role(name, mount_point=mount_point, **kwargs)
        return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_identity_group
version_added: "3.17.7"
short_description: Hashicorp Vault identity group configuration module
description:
    - Module to configure identity groups in Hashicorp Vault.
options:
    mount_point:
        description:
            - location where this method/backend is mounted. also known as "path"
        default: identity
    name:
        description:
            - name of the group
        default: None
    id:
        description:
            - ID of the group
        default: None
    group_type:
        description:
            - Type of the group, internal or external. Defaults to internal
        default: internal
    metadata:
        description:
            - metadata to be associated with the group
        default: None
    policies:
        description:
            - policies to be tied to the group
        default: None
    member_group_ids:
        description:
            - group IDs to be assigned as group members
        default: None
    member_entity_ids:
        description:
            - entity IDs to be assigned as group members
        default: None
    state:
        description:
            - whether create/update or delete the entity
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_identity_group:
        name: 'my-group'
        policies:
            - 'my-policy'
        member_group_ids:
            - 'group-id-xxxx'
        member_entity_ids:
            - 'entity-id-xxxx'
        metadata:
            'department': 'ops'
        token: "{{ vault_token }}"
        url: "{{ vault_url }}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=False, type='str', default=None)
    argspec['id'] = dict(required=False, type='str', default=None)
    argspec['group_type'] = dict(required=False, type='str', default='internal')
    argspec['mount_point'] = dict(required=False, type='str', default='identity')
    argspec['metadata'] = dict(required=False, type='dict', default={})
    argspec['policies'] = dict(required=False, type='list', default=[])
    argspec['member_group_ids'] = dict(required=False, type='list', default=None)
    argspec['member_entity_ids'] = dict(required=False, type='list', default=None)
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_identity_group(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def hashivault_identity_group_update(group_details, client, group_id, group_name, group_type, group_metadata,
                                     group_policies, group_member_group_ids, group_member_entity_ids, mount_point):
    changed = False

    # if the groups were created without any entity members, group members, or policies,
    # then vault will return null for each respectively
    # if they were created with these and then all were removed it returns an empty list
    # for each respectively. The below is required to account for this
    # existing member_group_ids
    if group_details['member_group_ids'] is not None and group_member_group_ids is not None:
        if set(group_details['member_group_ids']) != set(group_member_group_ids):
            changed = True
    # new member_group_ids and none existing
    elif group_member_group_ids is not None and len(group_member_group_ids) > 0:
        changed = True

    # existing policies
    if group_details['policies'] is not None:
        if set(group_details['policies']) != set(group_policies):
            changed = True
    # new policies and none existing
    elif len(group_policies) > 0:
        changed = True

    # existing member_entity_ids
    if group_details['member_entity_ids'] is not None and group_member_entity_ids is not None:
        if set(group_details['member_entity_ids']) != set(group_member_entity_ids):
            changed = True
    # new member_entity_ids and none existing
    elif group_member_entity_ids is not None and len(group_member_entity_ids) > 0:
        changed = True

    # existing metadata
    if group_details['metadata'] is not None:
        if group_details['metadata'] != group_metadata:
            changed = True
    # new metadata and none existing
    elif len(group_metadata) > 0:
        changed = True

    if group_details['name'] != group_name or group_details['type'] != group_type or changed:
        try:
            response = client.secrets.identity.update_group(
                group_id=group_id,
                name=group_name,
                group_type=group_type,
                metadata=group_metadata,
                policies=group_policies,
                member_group_ids=group_member_group_ids,
                member_entity_ids=group_member_entity_ids,
                mount_point=mount_point
            )
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        if response.status_code == 204:
            return {'changed': True}
        return {'changed': True, 'data': response}
    return {'changed': False}


def hashivault_identity_group_create_or_update(params):
    client = hashivault_auth_client(params)
    group_name = params.get('name')
    group_id = params.get('id')
    group_type = params.get('group_type')
    mount_point = params.get('mount_point')
    group_metadata = params.get('metadata')
    group_policies = params.get('policies')
    group_member_group_ids = params.get('member_group_ids')
    group_member_entity_ids = params.get('member_entity_ids')

    if group_id is not None:
        try:
            group_details = client.secrets.identity.read_group(group_id=group_id)
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        return hashivault_identity_group_update(group_details['data'], client, group_id, group_name, group_type,
                                                group_metadata, group_policies, group_member_group_ids,
                                                group_member_entity_ids, mount_point)
    elif group_name is not None:
        try:
            group_details = client.secrets.identity.read_group_by_name(name=group_name)
        except Exception:
            response = client.secrets.identity.create_or_update_group_by_name(
                name=group_name,
                group_type=group_type,
                metadata=group_metadata,
                policies=group_policies,
                member_group_ids=group_member_group_ids,
                member_entity_ids=group_member_entity_ids,
                mount_point=mount_point
            )
            from requests.models import Response
            if isinstance(response, Response):
                response = response.json()
            return {'changed': True, 'data': response['data']}
        return hashivault_identity_group_update(group_details['data'], client, group_name=group_name,
                                                group_id=group_details['data']['id'],
                                                group_type=group_type,
                                                group_metadata=group_metadata,
                                                group_policies=group_policies,
                                                group_member_group_ids=group_member_group_ids,
                                                group_member_entity_ids=group_member_entity_ids,
                                                mount_point=mount_point)
    return {'failed': True, 'msg': "Either name or id must be provided"}


def hashivault_identity_group_delete(params):
    client = hashivault_auth_client(params)
    group_id = params.get('id')
    group_name = params.get('name')

    if group_id is not None:
        try:
            client.secrets.identity.read_group(group_id=group_id)
        except Exception:
            return {'changed': False}
        client.secrets.identity.delete_group(group_id=group_id)
        return {'changed': True}
    elif group_name is not None:
        try:
            client.secrets.identity.read_group_by_name(name=group_name)
        except Exception:
            return {'changed': False}
        client.secrets.identity.delete_group_by_name(name=group_name)
        return {'changed': True}
    return {'failed': True, 'msg': "Either name or id must be provided"}


@hashiwrapper
def hashivault_identity_group(params):
    state = params.get('state')
    if state == 'present':
        return hashivault_identity_group_create_or_update(params)
    elif state == 'absent':
        return hashivault_identity_group_delete(params)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_azure_auth_role
version_added: "3.17.7"
short_description: Hashicorp Vault azure secret engine role
description:
    - Module to define a Azure role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: azure
    name:
        description:
            - name of the role in vault
    policies:
        description:
            - name of policies in vault to assign to role
    token_ttl:
        description:
            - The TTL period of tokens issued using this role in seconds.
    token_max_ttl:
        description:
            - The maximum allowed lifetime of tokens issued in seconds using this role.
    token_period:
        description:
            - If set, indicates that the token generated using this role should never expire. The token should be
              renewed within the duration specified by this value. At each renewal, the token's TTL will be set to the
              value of this parameter.
    bound_service_principal_ids:
        description:
            - The list of Service Principal IDs that login is restricted to.
    bound_group_ids:
        description:
            - The list of group ids that login is restricted to.
    bound_locations:
        description:
            - The list of locations that login is restricted to.
    bound_subscription_ids:
        description:
            - The list of subscription IDs that login is restricted to.
    bound_resource_groups:
        description:
            - The list of resource groups that login is restricted to.
    bound_scale_sets:
        description:
            - The list of scale set names that the login is restricted to.
    role_file:
        description:
            - File with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_azure_auth_role:
        name: "test"
        policies: ["test"]
        bound_subscription_ids: ["6a1d5988-5917-4221-b224-904cd7e24a25"]
        num_uses: 3

    - hashivault_azure_auth_role:
        name: test
        role_file: /users/drewbuntu/my-auth-role.json
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['role_file'] = dict(required=False, type='str')
    argspec['policies'] = dict(required=False, type='list')
    argspec['mount_point'] = dict(required=False, type='str', default='azure')
    argspec['token_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_max_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_period'] = dict(required=False, type='int', default=0)
    argspec['bound_service_principal_ids'] = dict(required=False, type='list', default=[])
    argspec['bound_group_ids'] = dict(required=False, type='list', default=[])
    argspec['bound_locations'] = dict(required=False, type='list', default=[])
    argspec['bound_subscription_ids'] = dict(required=False, type='list', default=[])
    argspec['bound_resource_groups'] = dict(required=False, type='list', default=[])
    argspec['bound_scale_sets'] = dict(required=False, type='list', default=[])
    argspec['num_uses'] = dict(required=False, type='int', default=0)

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_azure_auth_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_azure_auth_role(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')
    role_file = params.get('role_file')
    name = params.get('name').strip('/')
    state = params.get('state')
    desired_state = dict()
    exists = False
    changed = False

    # if azure_role_file is set, set azure_role to contents
    # else assume azure_role is set and use that value
    if role_file:
        desired_state = json.loads(open(params.get('role_file'), 'r').read())
    else:
        desired_state['policies'] = params.get('policies')
        desired_state['ttl'] = params.get('token_ttl')
        desired_state['max_ttl'] = params.get('token_max_ttl')
        desired_state['period'] = params.get('token_period')
        desired_state['bound_service_principal_ids'] = params.get('bound_service_principal_ids')
        desired_state['bound_group_ids'] = params.get('bound_group_ids')
        desired_state['bound_locations'] = params.get('bound_locations')
        desired_state['bound_subscription_ids'] = params.get('bound_subscription_ids')
        desired_state['bound_resource_groups'] = params.get('bound_resource_groups')
        desired_state['bound_scale_sets'] = params.get('bound_scale_sets')
        desired_state['num_uses'] = params.get('num_uses')

    # check if role exists
    try:
        existing_roles = client.auth.azure.list_roles(mount_point=mount_point)
        if name in existing_roles['keys']:
            # this role exists
            exists = True
    except Exception:
        # no roles exist yet
        pass

    if not exists and state == 'present':
        changed = True

    # compare current_state to desired_state
    if exists and state == 'present' and not changed:
        current_state = client.auth.azure.read_role(name=name)
        # Map a couple elements
        if 'ttl' not in current_state and 'token_ttl' in current_state:
            current_state['ttl'] = current_state['token_ttl']
        if 'max_ttl' not in current_state and 'token_max_ttl' in current_state:
            current_state['max_ttl'] = current_state['token_max_ttl']
        if 'period' not in current_state and 'token_period' in current_state:
            current_state['period'] = current_state['token_period']
        for k, v in desired_state.items():
            if k in current_state and v != current_state[k]:
                changed = True
    elif exists and state == 'absent':
        changed = True

    # make the changes!
    if changed and state == 'present' and not module.check_mode:
        client.auth.azure.create_role(name=name, mount_point=mount_point, **desired_state)

    elif changed and state == 'absent' and not module.check_mode:
        client.auth.azure.delete_role(name=name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_approle_role_secret_get
version_added: "3.8.0"
short_description: Hashicorp Vault approle role secret id get module
description:
    - Module to get a approle role secret id from Hashicorp Vault.
options:
    name:
        description:
            - role name.
    mount_point:
        description:
            - mount point for role
        default: approle
    secret:
        description:
            - secret id.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_approle_role_secret_get:
        name: 'ashley'
        secret: 'ec4bedee-e44b-c096-9ac8-1600e52ed8f8'
      register: 'vault_approle_role_secret_get'
    - debug: msg="Role secret is {{vault_approle_role_secret_get.secret}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='approle')
    argspec['secret'] = dict(required=True, type='str')
    module = hashivault_init(argspec)
    result = hashivault_approle_role_secret_get(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_approle_role_secret_get(params):
    try:
        name = params.get('name')
        mount_point = params.get('mount_point')
        secret = params.get('secret')
        client = hashivault_auth_client(params)
        response = client.get_role_secret_id(name, secret, mount_point=mount_point)
        if type(response) is not dict and response.status_code == 204:  # No content
            return {'secret': {}, 'status': 'absent'}
        else:
            return {'secret': response['data'], 'response': response, 'status': 'present'}
    except Exception as e:
        return {'failed': True, 'msg': str(e)}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['deprecated'], 'alternative': 'Use M(hashivault_secret_engine) instead.',
                    'why': 'This module does not fit the standard pattern',
                    'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_secret_disable
version_added: "2.2.0"
short_description: Hashicorp Vault secret disable module
description:
    - Module to disable secret backends in Hashicorp Vault. Use hashivault_secret_engine instead.
options:
    name:
        description:
            - name of secret backend
    backend:
        description:
            - type of secret backend
    description:
        description:
            - description of secret backend
    config:
        description:
            - config of secret backend
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_secret_disable:
        name: "ephemeral"
        backend: "generic"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    module = hashivault_init(argspec)
    result = hashivault_secret_disable(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_secret_disable(params):
    client = hashivault_auth_client(params)
    name = params.get('name')
    client.sys.disable_secrets_engine(name)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_ca_set
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Submit CA Information
description:
    - This module allows submitting the CA information for the backend via a PEM file containing the CA certificate and
      its private key, concatenated.
    - May optionally append additional CA certificates. Useful when creating an intermediate CA to ensure a full chain
      is returned when signing or generating certificates.
    - Not needed if you are generating a self-signed root certificate, and not used if you have a signed intermediate
      CA certificate with a generated key, use the hashivault_pki_set_signed for that. If you have already set a
      certificate and key, they will be overridden.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    pem_bundle:
        required: true
        type: str
        description:
        - Specifies the key and certificate concatenated in PEM format.
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_ca_set:
        pem_bundle: '-----BEGIN RSA PRIVATE KEY-----\n...\n-----END CERTIFICATE-----'
'''


def main():
    argspec = hashivault_argspec()
    argspec['pem_bundle'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_ca_set(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_ca_set(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')
    pem_bundle = params.get('pem_bundle')

    if module.check_mode:
        return {'changed': True}

    result = {'changed': True}
    data = client.secrets.pki.submit_ca_information(pem_bundle=pem_bundle, mount_point=mount_point)
    if data:
        from requests.models import Response
        if isinstance(data, Response):
            result['data'] = data.text
        else:
            result['data'] = data

    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Vault Lookup Plugin
#
# A simple example of using the vault plugin in a role:
#    ---
#    - debug: msg="{{lookup('vault', 'ldapadmin', 'password', mount_point='kv2')}}"
#
# The plugin must be run with VAULT_ADDR and VAULT_TOKEN set and
# exported.
#
# The plugin can be run manually for testing:
#     python ansible/plugins/lookup/hashivault.py ldapadmin password
#
import os
import sys

from ansible.errors import AnsibleError
from ansible.module_utils.basic import AnsibleFallbackNotFound
from ansible.plugins.lookup import LookupBase

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_read


class LookupModule(LookupBase):

    def _get_params(self, argspec, terms, kwargs):
        params = {}
        for arg in argspec:
            spec = argspec[arg]
            if arg in kwargs:
                params[arg] = kwargs[arg]
            elif 'default' in spec:
                params[arg] = spec.get('default')
            elif 'fallback' in spec:
                fall_func, fall_args = spec.get('fallback')
                try:
                    params[arg] = fall_func(*tuple(fall_args))
                except AnsibleFallbackNotFound:
                    pass

        params['secret'] = terms[0]
        try:
            params['key'] = terms[1]
        except IndexError:
            params['key'] = None
        return params

    def run(self, terms, variables=None, **kwargs):
        # self._display.v('Running lookup')
        argspec = hashivault_argspec()
        argspec['version'] = dict(required=False, type='int', default=1)
        argspec['mount_point'] = dict(required=False, type='str', default='secret')
        argspec['secret'] = dict(required=True, type='str')
        argspec['key'] = dict(required=False, type='str')
        argspec['default'] = dict(required=False, default=None, type='str')
        params = self._get_params(argspec, terms, kwargs)
        # self._display.v('ARGSPEC: ' + str(argspec))
        # self._display.v('KWARGS: ' + str(kwargs))
        # self._display.v('PARAMS: ' + str(params))
        result = hashivault_read(params=params)
        if 'value' not in result:
            path = terms[0]
            try:
                key = '/' + terms[1]
            except IndexError:
                key = ''
            raise AnsibleError('Error reading vault %s%s: %s\n%s' % (path, key, result.get('msg', 'msg not set'),
                                                                     result.get('stack_trace', '')))
        return [result['value']]


def main(argv=sys.argv[1:]):
    if len(argv) < 1:
        print("Usage: hashivault.py path [key]")
        return -1
    print(LookupModule().run(argv, {})[0])
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_approle_role_secret
version_added: "4.0.0"
short_description: Hashicorp Vault approle role secret id manager
description:
    - Create, update and delete approle secrets.
options:
    state:
        description:
            - present or absent
        default: present
    name:
        description:
            - secret name.
    mount_point:
        description:
            - mount point for role
        default: approle
    secret:
        description:
            - Secret id to be added or deleted. (was secret_id)
    cidr_list:
        description:
            - Comma-separated string or list of CIDR blocks.
    metadata:
        description:
            - Metadata to be tied to the secret.
    wrap_ttl:
        description:
            - Wrap TTL.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_approle_role_secret:
        name: ashley
        state: present
      register: vault_approle_role_secret_create
    - debug: msg="Role secret id is {{vault_approle_role_secret_create.id}}"

- hosts: localhost
  tasks:
    - hashivault_approle_role_secret:
        name: robert
        state: present
        secret: '{{ lookup("password", "/dev/null length=32 chars=ascii_letters,digits") }}'
      register: vault_approle_role_custom_secret_create
    - debug: msg="Role custom secret id is {{vault_approle_role_custom_secret_create.id}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    argspec['name'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='approle')
    argspec['cidr_list'] = dict(required=False, type='str')
    argspec['metadata'] = dict(required=False, type='dict')
    argspec['wrap_ttl'] = dict(required=False, type='str')
    argspec['secret'] = dict(required=False, type='str')
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_approle_role_secret(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_approle_role_secret(module):
    params = module.params
    state = params.get('state')
    name = params.get('name')
    mount_point = params.get('mount_point')

    client = hashivault_auth_client(params)
    if state == 'present':
        custom_secret_id = params.get('secret')
        if custom_secret_id is None:
            custom_secret_id = params.get('secret_id')  # deprecated
        cidr_list = params.get('cidr_list')
        metadata = params.get('metadata')
        wrap_ttl = params.get('wrap_ttl')
        if custom_secret_id is not None:
            if module.check_mode:
                try:
                    client.get_role_secret_id(name, custom_secret_id, mount_point=mount_point)
                except Exception:
                    return {'changed': True}
                return {'changed': False}
            result = client.create_role_custom_secret_id(role_name=name,
                                                         mount_point=mount_point,
                                                         secret_id=custom_secret_id,
                                                         meta=metadata)
        else:
            if module.check_mode:
                return {'changed': True}
            result = client.create_role_secret_id(role_name=name,
                                                  mount_point=mount_point,
                                                  meta=metadata,
                                                  cidr_list=cidr_list,
                                                  wrap_ttl=wrap_ttl)

        if wrap_ttl is None:
            response_key = 'data'
        else:
            response_key = 'wrap_info'

        return {'changed': True, response_key: result.get(response_key, {})}
    elif state == 'absent':
        secret = params.get('secret')
        if module.check_mode:
            try:
                client.get_role_secret_id(name, secret, mount_point=mount_point)
            except Exception:
                return {'changed': False}
            return {'changed': True}
        else:
            client.delete_role_secret_id(name, secret, mount_point=mount_point)
        return {'changed': True}
    else:
        return {'failed': True, 'msg': 'Unkown state value: {}'.format(state)}


if __name__ == '__main__':
    main()



########################################################################
#
# Developed for AT&T by Nicholas Gibson, August 2017
#
# Action plugin for hashivault_read_to_file module.
#
# Reads file/secret from Vault  on localhost using hashivault_read module.
# Decodes from base64 and stores file on remote host using copy module.
#
########################################################################

import base64 as base64encode
import os
import tempfile

from ansible.playbook.play_context import PlayContext
from ansible.plugins.action import ActionBase
from ansible.utils.vars import merge_hash


class ActionModule(ActionBase):

    # load and return ansible copy action plugin
    # copied from `ansible/plugins/action/template.py`
    def _get_copy_action_plugin(self, connection):
        return (self._shared_loader_obj.action_loader.get(
            'copy',
            task=self._task.copy(),
            connection=connection,
            play_context=self._play_context,
            loader=self._loader,
            templar=self._templar,
            shared_loader_obj=self._shared_loader_obj))

    def run(self, tmp=None, task_vars=None):

        if task_vars is None:
            task_vars = dict()

        results = super(ActionModule, self).run(tmp, task_vars)

        args = self._task.args.copy()

        dest = args.pop('dest', None)
        force = args.pop('force', True)
        base64 = args.pop('base64', True)

        new_module_args = {
            'dest': dest,
            'force': force,
        }
        copy_attributes = [
            'attributes',
            'backup',
            'checksum',
            'delimiter',
            'directory_mode',
            'follow',
            'group',
            'local_follow',
            'mode',
            'owner',
            'regexp',
            'selevel',
            'serole',
            'setype',
            'seuser',
            'unsafe_writes',
            'validate',
        ]
        for attribute in copy_attributes:
            value = args.pop(attribute, None)
            if value is not None:
                new_module_args[attribute] = value

        become = self._play_context.become
        become_method = self._play_context.become_method

        old_connection = self._connection
        self._connection = self._shared_loader_obj.connection_loader.get('local', PlayContext(),
                                                                         old_connection._new_stdin)
        self._play_context.become = False
        self._play_context.become_method = None

        results = merge_hash(
            results,
            # executes hashivault_read module on localhost
            self._execute_module(module_name='hashivault_read', tmp=tmp, task_vars=task_vars, module_args=args)
        )

        if 'failed' in results and results['failed'] is True:
            return results

        content = results.pop('value', None)

        if content is None:
            results['failed'] = True
            results['rc'] = 1
            results['msg'] = u'Could not find file `%s` in secret `%s`' % (args['key'], args['secret'])
            return results

        # write to temp file on ansible host to copy to remote host
        local_tmp = tempfile.NamedTemporaryFile(delete=False)
        if base64:
            try:
                content = base64encode.b64decode(content)
            except Exception as ex:
                results['failed'] = True
                results['rc'] = 1
                secret_key = str(args.pop('secret', 'secret')) + "/" + str(args.pop('key', ''))
                results['msg'] = u'Error base64 decoding secret %s: %s' % (secret_key, str(ex))
                return results
        else:
            try:
                import sys
                if sys.version_info[0] > 2:
                    content = bytes(content, 'utf-8')
                else:
                    content = bytes(content)
            except Exception as ex:
                results['failed'] = True
                results['rc'] = 1
                secret_key = str(args.pop('secret', 'secret')) + "/" + str(args.pop('key', ''))
                results['msg'] = u'Error preparing utf-8 secret %s: %s' % (secret_key, str(ex))
                return results
        local_tmp.write(content)
        local_tmp.close()
        new_module_args['src'] = local_tmp.name

        self._update_module_args('copy', new_module_args, task_vars)

        # `copy` module uses an action plugin, so we have to execute
        # the plugin instead of directly executing the module
        copy_action = self._get_copy_action_plugin(old_connection)
        copy_action._task.args = new_module_args
        copy_action._play_context.become = become
        copy_action._play_context.become_method = become_method

        results = merge_hash(
            results,
            # executes copy action plugin/module on remote host
            copy_action.run(task_vars=task_vars)
        )

        # remove temp file
        os.unlink(local_tmp.name)

        if force is False and results['changed'] is False:
            results['failed'] = True
            results['rc'] = 1
            results['msg'] = u'File %s already exists. Use `force: true` to overwrite' % dest

        return results



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['deprecated'], 'alternative': 'Use M(hashivault_policy) instead.',
                    'why': 'This module does not fit the standard pattern',
                    'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_policy_set
version_added: "2.1.0"
short_description: Hashicorp Vault policy set module
description:
    - Module to set a policy in Hashicorp Vault. Use hashivault_policy instead.
options:
    name:
        description:
            - policy name.
    rules:
        description:
            - policy rules.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_policy_set:
      rules: '{{rules}}'
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['rules'] = dict(required=True, type='str')
    argspec['rules_file'] = dict(required=False, type='bool', default=False)
    module = hashivault_init(argspec)
    result = hashivault_policy_set(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_policy_set(params):
    client = hashivault_auth_client(params)
    name = params.get('name')
    rules = params.get('rules')
    rules_file = params.get('rules_file')
    if rules_file:
        try:
            rules = open(rules, 'r').read()
        except Exception as e:
            return {'changed': False, 'failed': True, 'msg': 'Error opening rules file <%s>: %s' % (rules, str(e))}
    current = client.get_policy(name)
    if current == rules:
        return {'changed': False}
    client.sys.create_or_update_policy(name, rules)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
from hvac.exceptions import InvalidPath

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_auth_ldap
version_added: "3.17.7"
short_description: Hashicorp Vault ldap configuration module
description:
    - Module to configure the LDAP authentication method in Hashicorp Vault.
options:
    mount_point:
        description:
            - location where this auth_method is mounted. also known as "path"
    ldap_url:
        description:
            - The LDAP server to connect to. Examples: ldap://ldap.myorg.com
        default: ldap://127.0.0.1
    case_sensitive_names:
        description:
            - If set, user and group names assigned to policies within the backend will be case sensitive. Otherwise,
              names will be normalized to lower case. Case will still be preserved when sending the username to the LDAP
              server at login time; this is only for matching local user/group definitions.
        default: False
    starttls:
        description:
            - If true, issues a StartTLS command after establishing an unencrypted connection
        default: False
    tls_min_version:
        description:
            - Minimum TLS version to use. Accepted values are tls10, tls11 or tls12
        default: tls12
    tls_max_version:
        description:
            - Maximum TLS version to use. Accepted values are tls10, tls11 or tls12
        default: tls12
    insecure_tls:
        description:
            -  If true, skips LDAP server SSL certificate verification
        default: False
    certificate:
        description:
            - CA certificate to use when verifying LDAP server certificate, must be x509 PEM encoded
        default: ''
    bind_dn:
        description:
         - Distinguished name of object to bind when performing user search.
           Example cn=vault,ou=Users,dc=example,dc=com
        default: ''
    bind_pass:
        description:
            - Password to use along with binddn when performing user search
        default: None
    user_dn:
        description:
            - Base DN under which to perform user search. Example: ou=Users,dc=example,dc=com
        default: ''
    user_attr:
        description:
         - Attribute on user attribute object matching the username passed when authenticating.
           Examples sAMAccountName, cn, uid
        default: cn
    discover_dn:
        description:
         - Use anonymous bind to discover the bind DN of a user
        default: False
    deny_null_bind:
        description:
         - This option prevents users from bypassing authentication when providing an empty password
        default: True
    upn_domain:
        description:
         - The userPrincipalDomain used to construct the UPN string for the authenticating user
        default: ''
    group_filter:
        description:
            - Go template used when constructing the group membership query. The template can access the following
              context variables [UserDN, Username]
        default: (|(memberUid={{.Username}})(member={{.UserDN}})(uniqueMember={{.UserDN}}))
    group_attr:
        description:
            - LDAP attribute to follow on objects returned by groupfilter in order to enumerate user group membership
        default: 'cn'
    group_dn:
        description:
            - LDAP search base to use for group membership search
        default: ''
    token_ttl:
        description:
            - The incremental lifetime for generated tokens
        default: ''
    token_max_ttl:
        description:
            - The maximum lifetime for generated tokens
        default: ''
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_auth_ldap:
        user_dn: "{{ auth_ldap_userdn }}"
        group_dn: "{{ auth_ldap_groupdn }}"
        bind_dn: "{{ auth_ldap_binddn }}"
        ldap_url: "{{ auth_ldap_url }}"
        insecure_tls: "{{ auth_ldap_insecure_tls }}"
        group_filter: "{{ auth_ldap_groupfilter }}"
        upn_domain: "{{ auth_ldap_upndomain }}"
'''


def main():
    # separate long default value to pass linting
    default_group_filter = '(|(memberUid={{.Username}})(member={{.UserDN}})(uniqueMember={{.UserDN}}))'
    argspec = hashivault_argspec()
    argspec['description'] = dict(required=False, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='ldap')
    argspec['ldap_url'] = dict(required=False, type='str', default='ldap://127.0.0.1')
    argspec['case_sensitive_names'] = dict(required=False, type='bool', default=False)
    argspec['starttls'] = dict(required=False, type='bool', default=False)
    argspec['tls_min_version'] = dict(required=False, type='str', default='tls12')
    argspec['tls_max_version'] = dict(required=False, type='str', default='tls12')
    argspec['insecure_tls'] = dict(required=False, type='bool', default=False)
    argspec['certificate'] = dict(required=False, type='str', default='')
    argspec['bind_dn'] = dict(required=False, type='str', default='')
    argspec['bind_pass'] = dict(required=False, type='str', default=None, no_log=True)
    argspec['user_attr'] = dict(required=False, type='str', default='cn')
    argspec['user_dn'] = dict(required=False, type='str', default='')
    argspec['discover_dn'] = dict(required=False, type='bool', default=False)
    argspec['deny_null_bind'] = dict(required=False, type='bool', default=True)
    argspec['upn_domain'] = dict(required=False, type='str', default='')
    argspec['group_filter'] = dict(required=False, type='str', default=default_group_filter)
    argspec['group_attr'] = dict(required=False, type='str', default='cn')
    argspec['group_dn'] = dict(required=False, type='str', default='')
    argspec['use_token_groups'] = dict(required=False, type='bool', default=False)
    argspec['token_ttl'] = dict(required=False, type='str', default='')
    argspec['token_max_ttl'] = dict(required=False, type='str', default='')

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_auth_ldap(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_auth_ldap(module):
    params = module.params
    client = hashivault_auth_client(params)
    changed = False
    desired_state = dict()
    desired_state['mount_point'] = params.get('mount_point')
    desired_state['url'] = params.get('ldap_url')
    desired_state['case_sensitive_names'] = params.get('case_sensitive_names')
    desired_state['starttls'] = params.get('starttls')
    desired_state['tls_min_version'] = params.get('tls_min_version')
    desired_state['tls_max_version'] = params.get('tls_max_version')
    desired_state['insecure_tls'] = params.get('insecure_tls')
    desired_state['certificate'] = params.get('certificate')
    desired_state['bind_dn'] = params.get('bind_dn')
    desired_state['bind_pass'] = params.get('bind_pass')
    desired_state['user_attr'] = params.get('user_attr')
    desired_state['user_dn'] = params.get('user_dn')
    desired_state['discover_dn'] = params.get('discover_dn')
    desired_state['deny_null_bind'] = params.get('deny_null_bind')
    desired_state['upn_domain'] = params.get('upn_domain')
    desired_state['group_filter'] = params.get('group_filter')
    desired_state['group_attr'] = params.get('group_attr')
    desired_state['group_dn'] = params.get('group_dn')
    desired_state['use_token_groups'] = params.get('use_token_groups')
    desired_state['token_ttl'] = params.get('token_ttl')
    desired_state['token_max_ttl'] = params.get('token_max_ttl')

    # if bind pass is None, remove it from desired state since we can't compare
    if desired_state['bind_pass'] is None:
        del desired_state['bind_pass']

    # check current config
    current_state = dict()
    try:
        result = client.auth.ldap.read_configuration(
            mount_point=desired_state['mount_point'])['data']
        # some keys need to be remapped to match desired state (and HVAC implementation)
        current_state['discover_dn'] = result['discoverdn']
        current_state['group_attr'] = result['groupattr']
        current_state['user_attr'] = result['userattr']
        current_state['group_dn'] = result['groupdn']
        current_state['upn_domain'] = result['upndomain']
        current_state['group_filter'] = result['groupfilter']
        current_state['case_sensitive_names'] = result['case_sensitive_names']
        current_state['certificate'] = result['certificate']
        current_state['tls_max_version'] = result['tls_max_version']
        current_state['tls_min_version'] = result['tls_min_version']
        current_state['insecure_tls'] = result['insecure_tls']
        current_state['deny_null_bind'] = result['deny_null_bind']
        current_state['user_dn'] = result['userdn']
        current_state['bind_dn'] = result['binddn']
        current_state['use_token_groups'] = result['use_token_groups']
        current_state['url'] = result['url']
        current_state['starttls'] = result['starttls']
        current_state['token_ttl'] = result['token_ttl']
        current_state['token_max_ttl'] = result['token_max_ttl']
    except InvalidPath:
        pass

    # check if current config matches desired config values, if they match, set changed to false to prevent action
    for k, v in current_state.items():
        if v != desired_state[k]:
            changed = True

    # if configs dont match and checkmode is off, complete the change
    if changed and not module.check_mode:
        client.auth.ldap.configure(**desired_state)

    return {'changed': changed}


if __name__ == '__main__':
    main()



class ModuleDocFragment(object):
    # Standard documentation
    DOCUMENTATION = r'''
    requirements:
        - hvac>=0.10.1
        - ansible>=2.0.0
        - requests
    options:
        url:
            description:
                - url for vault
            default: to environment variable `VAULT_ADDR`
        ca_cert:
            description:
                - Path to a PEM-encoded CA cert file to use to verify the Vault server TLS certificate
            default: to environment variable `VAULT_CACERT`
        ca_path:
            description:
                - Path to a directory of PEM-encoded CA cert files to verify the Vault server TLS certificate. If
                  ca_cert is specified, its value will take precedence
            default: to environment variable `VAULT_CAPATH`
        client_cert:
            description:
                - Path to a PEM-encoded client certificate for TLS authentication to the Vault server
            default: to environment variable `VAULT_CLIENT_CERT`
        client_key:
            description:
                - Path to an unencrypted PEM-encoded private key matching the client certificate
            default: to environment variable `VAULT_CLIENT_KEY`
        verify:
            description:
                - If set, do not verify presented TLS certificate before communicating with Vault server. Setting this
                  variable is not recommended except during testing
            default: to environment variable `VAULT_SKIP_VERIFY`
        authtype:
            description:
                - authentication type
            default: token or environment variable `VAULT_AUTHTYPE`
            choices: ["token", "userpass", "github", "ldap", "approle"]
        login_mount_point:
            description:
                - authentication mount point
            default: value of authtype or environment varialbe `VAULT_LOGIN_MOUNT_POINT`
        token:
            description:
                - token for vault
            default: to environment variable `VAULT_TOKEN`
        username:
            description:
                - username to login to vault.
            default: to environment variable `VAULT_USER`
        password:
            description:
                - password to login to vault.
            default: to environment variable `VAULT_PASSWORD`
        aws_header:
            description:
                - X-Vault-AWS-IAM-Server-ID Header value to prevent replay attacks.
            default: to environment variable `VAULT_AWS_HEADER`
        namespace:
            description:
                - namespace for vault
            default: to environment variable VAULT_NAMESPACE
'''



#!/usr/bin/env python
from ansible.module_utils.hashivault import is_state_changed
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_auth_method
version_added: "3.17.7"
short_description: Hashicorp Vault auth module
description:
    - Module to enable or disable authentication methods in Hashicorp Vault.
options:
    method_type:
        description:
            - name of auth method. [required]
    description:
        description:
            - description of authenticator
    mount_point:
        description:
            - location where this auth_method will be mounted. also known as "path"
    state:
        description:
            - should auth mount be enabled or disabled
        default: enabled
    config:
        description:
            - configuration set on auth method. expects a dict
        default: "{'default_lease_ttl': 2764800, 'max_lease_ttl': 2764800, 'force_no_cache':False,
                  'token_type': 'default-service'}"
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_auth_method:
        method_type: userpass
'''

DEFAULT_TTL = 2764800
DEFAULT_CONFIG = {
    'default_lease_ttl': DEFAULT_TTL,
    'max_lease_ttl': DEFAULT_TTL,
    'force_no_cache': False,
    'token_type': 'default-service'
}


def main():
    argspec = hashivault_argspec()
    argspec['method_type'] = dict(required=True, type='str')
    argspec['description'] = dict(required=False, type='str', default='')
    argspec['state'] = dict(required=False, type='str', default='enabled',
                            choices=['enabled', 'disabled', 'enable', 'disable'])
    argspec['mount_point'] = dict(required=False, type='str', default=None)
    argspec['config'] = dict(required=False, type='dict',
                             default=DEFAULT_CONFIG)
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_auth_method(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_auth_method(module):
    params = module.params
    client = hashivault_auth_client(params)
    method_type = params.get('method_type')
    description = params.get('description')
    mount_point = params.get('mount_point')
    config = params.get('config')
    if params.get('state') in ['enabled', 'enable']:
        state = 'enabled'
    else:
        state = 'disabled'
    exists = False
    changed = False
    create = False

    if mount_point is None:
        mount_point = method_type

    try:
        result = client.sys.list_auth_methods()
        auth_methods = result.get('data', result)
        if (mount_point + u"/") in auth_methods:
            exists = True
    except Exception:
        pass

    if state == 'enabled' and not exists:
        changed = True
        create = True
    elif state == 'disabled' and exists:
        changed = True
    elif exists and state == 'enabled':
        current_state = auth_methods[mount_point + u"/"]
        changed = description != current_state['description'] or is_state_changed(config, current_state['config'])

    if module.check_mode:
        return {'changed': changed, 'created': create, 'state': state}
    if not changed:
        return {'changed': changed, 'created': False, 'state': state}

    if state == 'enabled':
        if create:
            client.sys.enable_auth_method(method_type, description=description, path=mount_point, config=config)
        else:
            client.sys.tune_auth_method(description=description, path=mount_point, **config)
    if state == 'disabled':
        client.sys.disable_auth_method(path=mount_point)

    return {'changed': changed, 'created': create}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import is_state_changed
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

DEFAULT_TTL = 2764800
ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_secret_engine
version_added: "3.17.8"
short_description: Hashicorp Vault secret enable/disable module
description:
    - Module to enable secret backends in Hashicorp Vault.
options:
    name:
        description:
            - name of secret backend
    backend:
        description:
            - type of secret backend
    description:
        description:
            - description of secret backend
    config:
        type: dict
        description:
            - config of secret backend
        suboptions:
            default_lease_ttl:
                type: str or int
                default: 2764800 (= 768 hours)
                description:
                    - Specifies the default lease duration. It can be either an int (representing the number of
                      seconds) or duration string with s, m, h or d suffix.
            max_lease_ttl:
                type: str or int
                default: 2764800 (= 768 hours)
                description:
                    - Specifies the maximum lease duration. It can be either an int (representing the number of
                      seconds) or duration string with s, m, h or d suffix.
            force_no_cache:
                type: bool
                default: false
                description:
                    - Disable caching
            audit_non_hmac_request_keys:
                type: list
                description:
                    - List of keys that will not be HMAC'd by audit devices in the request data object.
            audit_non_hmac_response_keys:
                type: list
                description:
                    - List of keys that will not be HMAC'd by audit devices in the response data object.
            listing_visibility:
                type: str
                description:
                    - Specifies whether to show this mount in the UI-specific listing endpoint. Valid values are
                      "unauth" or "hidden". If not set, behaves like "hidden".
            passthrough_request_headers:
                type: list
                description:
                    - List of headers to whitelist and pass from the request to the plugin.
            allowed_response_headers:
                type: list
                description:
                    - List of headers to whitelist, allowing a plugin to include them in the response.
    state:
        description:
            - state of secret backend. choices: enabled, present, disabled, absent
    options:
        description:
            - Specifies mount type specific options that are passed to the backend. NOT included unless backend == kv
    cas_required:
        description:
            - Check and set required for secret engine for kv version 2 only
    max_versions:
        description:
            - Max versions for secret engine for kv version 2 only
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_secret_engine:
        name: ephemeral
        backend: generic
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['backend'] = dict(required=False, type='str', default='')
    argspec['description'] = dict(required=False, type='str', default='')
    argspec['config'] = dict(required=False, type='dict', default={'default_lease_ttl': DEFAULT_TTL,
                                                                   'max_lease_ttl': DEFAULT_TTL,
                                                                   'force_no_cache': False})
    argspec['state'] = dict(required=False, type='str', choices=['present', 'enabled', 'absent', 'disabled'],
                            default='present')
    argspec['options'] = dict(required=False, type='dict', default={})
    argspec['cas_required'] = dict(required=False, type='bool')
    argspec['max_versions'] = dict(required=False, type='int')
    module = hashivault_init(argspec)
    result = hashivault_secret_engine(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def parse_duration(duration):
    if isinstance(duration, int):
        return duration
    elif not isinstance(duration, str):
        return DEFAULT_TTL

    if duration.endswith('d'):
        return int(duration[:-1]) * 60 * 60 * 24
    if duration.endswith('h'):
        return int(duration[:-1]) * 60 * 60
    if duration.endswith('m'):
        return int(duration[:-1]) * 60
    if duration.endswith('s'):
        return int(duration[:-1])
    if duration != "":
        return int(duration)

    return DEFAULT_TTL


@hashiwrapper
def hashivault_secret_engine(module):
    params = module.params
    client = hashivault_auth_client(params)
    name = params.get('name')
    backend = params.get('backend')
    description = params.get('description')
    config = params.get('config')
    if 'default_lease_ttl' in config:
        config['default_lease_ttl'] = parse_duration(config['default_lease_ttl'])
    if 'max_lease_ttl' in config:
        config['max_lease_ttl'] = parse_duration(config['max_lease_ttl'])
    if params.get('state') in ['present', 'enabled']:
        state = 'enabled'
    else:
        state = 'disabled'
    options = params.get('options')
    cas_required = params.get('cas_required')
    max_versions = params.get('max_versions')
    new_engine_configuration = {}
    if str(options.get('version', None)) == '2' or backend == 'kv-v2':
        if cas_required:
            new_engine_configuration['cas_required'] = cas_required
        if max_versions:
            new_engine_configuration['max_versions'] = max_versions
    current_state = dict()
    exists = False
    created = False
    changed = False

    if not backend:
        backend = name
    try:
        # does the mount exist already?
        configuration = client.sys.read_mount_configuration(path=name)
        current_state = configuration['data']
        exists = True
    except Exception:
        # doesn't exist
        pass

    # doesnt exist and should or does exist and shouldnt
    if (exists and state == 'disabled'):
        changed = True
    elif (not exists and state == 'enabled'):
        changed = True
    elif state == 'enabled':
        if 'version' in options:
            options['version'] = str(options['version'])

        if 'description' in current_state:
            if description != current_state['description']:
                changed = True
        if 'options' in current_state:
            current_options = current_state['options']
            if not changed:
                changed = is_state_changed(options, current_options)
        elif options:
            changed = True
        for key in config.keys():
            if key not in current_state:
                changed = True
            elif current_state[key] != config[key]:
                changed = True
        if new_engine_configuration and not changed:
            engine_configuration = client.secrets.kv.read_configuration(name)['data']
            if cas_required is not None and engine_configuration.get('cas_required', None) != cas_required:
                changed = True
            elif max_versions is not None and engine_configuration.get('max_versions', None) != max_versions:
                changed = True
            else:
                new_engine_configuration = {}

    # create
    if changed and not exists and state == 'enabled' and not module.check_mode:
        if backend == 'kv' or backend == 'kv-v2':
            client.sys.enable_secrets_engine(backend, description=description, path=name, config=config,
                                             options=options)
            if new_engine_configuration:
                client.secrets.kv.v2.configure(mount_point=name, cas_required=cas_required, max_versions=max_versions)
        else:
            client.sys.enable_secrets_engine(backend, description=description, path=name, config=config)
        created = True

    # update
    elif changed and exists and state == 'enabled' and not module.check_mode:
        if backend == 'kv' or backend == 'kv-v2':
            client.sys.tune_mount_configuration(path=name, description=description, options=options, **config)
            if new_engine_configuration:
                client.secrets.kv.v2.configure(mount_point=name, cas_required=cas_required, max_versions=max_versions)
        else:
            client.sys.tune_mount_configuration(path=name, description=description, **config)

    # delete
    elif changed and state == 'disabled' and not module.check_mode:
        client.sys.disable_secrets_engine(path=name)

    return {'changed': changed, 'created': created}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['deprecated'], 'alternative': 'Use M(hashivault_auth_method) instead.',
                    'why': 'This module does not fit the standard pattern',
                    'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_auth_enable
version_added: "2.2.0"
short_description: Hashicorp Vault auth enable module
description:
    - Use hashivault_auth_method instead.
options:
    name:
        description:
            - name of authenticator
    description:
        description:
            - description of authenticator
    mount_point:
        description:
            - location where this auth backend will be mounted
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_auth_enable:
        name: "userpass"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['description'] = dict(required=False, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default=None)
    module = hashivault_init(argspec)
    result = hashivault_auth_enable(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_auth_enable(params):
    client = hashivault_auth_client(params)
    name = params.get('name')
    description = params.get('description')
    mount_point = params.get('mount_point')
    result = client.sys.list_auth_methods()
    backends = result.get('data', result)
    path = (mount_point or name) + u"/"
    if path in backends:
        return {'changed': False}
    client.sys.enable_auth_method(name, description=description, path=mount_point)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['deprecated'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_approle_role_secret_create
version_added: "3.8.0"
short_description: Hashicorp Vault approle role secret id create module
description:
    - Module to get an approle role secret id from Hashicorp Vault in Pull mode
      or create custom approle role secret id in Push mode. Use hashivault_approle_role_secret instead.
options:
    name:
        description:
            - secret name.
    secret_id:
        description:
            - Custom SecretID to be attached to the role.
    cidr_list:
        description:
            - Comma-separated string or list of CIDR blocks.
    metadata:
        description:
            - Metadata to be tied to the secret.
    wrap_ttl:
        description:
            - Wrap TTL.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_approle_role_secret_create:
        name: 'ashley'
      register: 'vault_approle_role_secret_create'
    - debug: msg="Role secret id is {{vault_approle_role_secret_create.id}}"

- hosts: localhost
  tasks:
    - hashivault_approle_role_secret_create:
        name: 'robert'
        secret_id: '{{ lookup("password", "/dev/null length=32 chars=ascii_letters,digits") }}'
      register: 'vault_approle_role_custom_secret_create'
    - debug: msg="Role custom secret id is {{vault_approle_role_custom_secret_create.id}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['secret_id'] = dict(required=False, type='str')
    argspec['cidr_list'] = dict(required=False, type='str')
    argspec['metadata'] = dict(required=False, type='dict')
    argspec['wrap_ttl'] = dict(required=False, type='str')
    module = hashivault_init(argspec)
    result = hashivault_approle_role_secret_create(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_approle_role_secret_create(params):
    name = params.get('name')
    custom_secret_id = params.get('secret_id')
    cidr_list = params.get('cidr_list')
    metadata = params.get('metadata')
    wrap_ttl = params.get('wrap_ttl')

    client = hashivault_auth_client(params)

    if custom_secret_id is not None:
        result = client.create_role_custom_secret_id(role_name=name,
                                                     secret_id=custom_secret_id,
                                                     meta=metadata)
    else:
        result = client.create_role_secret_id(role_name=name, meta=metadata,
                                              cidr_list=cidr_list,
                                              wrap_ttl=wrap_ttl)
    return result['data']


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_token_renew
version_added: "3.11.0"
short_description: Hashicorp Vault token renew module
description:
    - Module to renew tokens in Hashicorp Vault.
options:
    renew_token:
        description:
            - token to renew if different from auth token
        default: to authentication token
    increment:
        description:
            - "Request a specific increment for renewal. Vault is not required to honor this request. If not supplied,\
             Vault will use the default TTL."
    wrap_ttl:
        description:
            - Indicates that the response should be wrapped in a cubbyhole token with the requested TTL.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - name: "Renew token"
      hashivault_token_renew:
        renew_token: "{{client_token}}"
        increment: "5m"
      register: "vault_token_renew"
'''


def main():
    argspec = hashivault_argspec()
    argspec['renew_token'] = dict(required=False, type='str', no_log=True)
    argspec['increment'] = dict(required=False, type='str', default=None)
    argspec['wrap_ttl'] = dict(required=False, type='int')
    module = hashivault_init(argspec)
    result = hashivault_token_renew(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_token_renew(params):
    client = hashivault_auth_client(params)
    renew_token = params.get('renew_token')
    increment = params.get('increment')
    if renew_token is None:
        renew_token = params.get('token')
    wrap_ttl = params.get('wrap_ttl')
    renew = client.renew_token(token=renew_token, increment=increment, wrap_ttl=wrap_ttl)
    return {'changed': True, 'renew': renew}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_crl_rotate
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Rotate CRLs
description:
    - This module forces a rotation of the CRL.
    - This can be used by administrators to cut the size of the CRL if it contains a number of certificates that have
      now expired, but has not been rotated due to no further certificates being revoked.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_crl_rotate:
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_crl_rotate(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_crl_rotate(module):
    params = module.params
    client = hashivault_auth_client(params)

    mount_point = params.get('mount_point').strip('/')

    failed = False

    # make the changes!
    if not module.check_mode:
        failed = not client.secrets.pki.rotate_crl(mount_point=mount_point).get('data').get('success')
    return {'failed': failed, 'changed': not failed,
            'msg': 'oops, something went wrong.' if failed else '', 'rc': 1 if failed else 0}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from hvac.exceptions import InvalidPath

from ansible.module_utils.hashivault import is_state_changed
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_secret
version_added: "0.1"
short_description: Hashicorp Vault write module
description:
    - Module to write to Hashicorp Vault.
options:
    state:
        description:
            - Update reads and overlays with values provided. Present reads and checks for changes and then overwrites
              with values provided. state of secret choices of present, update, or absent
    version:
        description:
            - version of the kv engine (int)
        default: 2
    mount_point:
        description:
            - secret mount point
        default: secret
    secret:
        description:
            - vault secret to write.
    data:
        description:
            - Keys and values to write.
    cas:
        description:
            - Check and set curent version 2 only
    permanent:
        description:
            - delete all versions and metadata for a given secret for kv engine version 2.
        default: false
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_secret:
        secret: giant
        data:
            foo: foe
            fie: fum
'''


def main():
    argspec = hashivault_argspec()
    argspec['state'] = dict(required=False, type='str', choices=['present', 'update', 'absent'],
                            default='present')
    argspec['version'] = dict(required=False, type='int', default=2)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(required=True, type='str')
    argspec['data'] = dict(required=False, default={}, type='dict', no_log=True)
    argspec['cas'] = dict(required=False, type='int')
    argspec['permanent'] = dict(required=False, type='bool', default=False)
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_secret(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_secret(module):
    params = module.params
    client = hashivault_auth_client(params)
    state = params.get('state')
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')
    data = params.get('data')
    cas = params.get('cas')
    permanent = params.get('permanent')

    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    result = {"changed": False, "rc": 0}
    try:
        if version == 2:
            read_data = client.secrets.kv.v2.read_secret_version(secret, mount_point=mount_point)
            read_data = read_data.get('data', {})
        else:
            read_data = client.secrets.kv.v1.read_secret(secret, mount_point=mount_point)
        read_data = read_data.get('data', {})
    except InvalidPath:
        if state not in ['present', 'update']:
            result['msg'] = u"Secret %s nonexistent" % secret_path
            return result
        read_data = {}
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        error_string = "%s(%s)" % (e.__class__.__name__, e)
        result['msg'] = u"Error %s reading %s" % (error_string, secret_path)
        return result

    if state == 'present' or state == 'update':
        if state == 'update':
            write_data = dict(read_data)
            write_data.update(data)
        else:
            write_data = data
        changed = is_state_changed(write_data, read_data)
        # result['write_data'] = write_data
        # result['read_data'] = read_data

        if not changed:
            result['msg'] = u"Secret %s unchanged" % secret_path
            return result
        if not module.check_mode:
            try:
                if version == 2:
                    client.secrets.kv.v2.create_or_update_secret(
                        mount_point=mount_point, cas=cas, path=secret, secret=write_data)
                else:
                    client.secrets.kv.v1.create_or_update_secret(
                        mount_point=mount_point, path=secret, secret=write_data)
            except Exception as e:
                result['rc'] = 1
                result['failed'] = True
                error_string = "%s(%s)" % (e.__class__.__name__, e)
                result['msg'] = u"Error %s writing %s" % (error_string, secret_path)
                return result

        result['msg'] = u"Secret %s written" % secret_path
        result['changed'] = True
    else:
        try:
            if version == 2:
                if permanent:
                    client.secrets.kv.v2.delete_metadata_and_all_versions(secret, mount_point=mount_point)
                else:
                    client.secrets.kv.v2.delete_latest_version_of_secret(secret, mount_point=mount_point)
            else:
                client.secrets.kv.v1.delete_secret(secret, mount_point=mount_point)
        except InvalidPath:
            result['msg'] = u"Secret %s nonexistent" % secret_path
            return result
        except Exception as e:
            result['rc'] = 1
            result['failed'] = True
            error_string = "%s(%s)" % (e.__class__.__name__, e)
            result['msg'] = u"Error %s deleting %s" % (error_string, secret_path)
            return result
        result['msg'] = u"Secret %s deleted" % secret_path
        result['changed'] = True
    return result


if __name__ == '__main__':
    main()
